@@ -216,7 +216,7 @@ Mutate state INFO with response metadata."
           ;; gptel--system-message is guaranteed to be a string
           (plist-put prompts-plist :system
                      `[(:type "text" :text ,gptel--system-message
-                        :cache_control (:type "ephemeral"))])
+                        :cache_control (:type "ephemeral" :ttl "1h"))])
         (plist-put prompts-plist :system gptel--system-message)))
     (when gptel-temperature
       (plist-put prompts-plist :temperature gptel-temperature))
@@ -229,13 +229,30 @@ Mutate state INFO with response metadata."
           (when (and (or (eq gptel-cache t) (memq 'tool gptel-cache))
                      (gptel--model-capable-p 'cache))
             (nconc (aref tools-array (1- (length tools-array)))
-                   '(:cache_control (:type "ephemeral")))))))
+                   '(:cache_control (:type "ephemeral" :ttl "1h")))))))
+    (when gptel--schema
+      (plist-put prompts-plist :tools
+                 (vconcat
+                  (list (gptel--parse-schema backend gptel--schema))
+                  (plist-get prompts-plist :tools)))
+      (plist-put prompts-plist :tool_choice
+                 `(:type "tool" :name ,gptel--ersatz-json-tool)))
     ;; Merge request params with model and backend params.
     (gptel--merge-plists
      prompts-plist
+     gptel--request-params
      (gptel-backend-request-params gptel-backend)
      (gptel--model-request-params  gptel-model))))
 
+(cl-defmethod gptel--parse-schema ((_backend gptel-anthropic) schema)
+  ;; Unlike the other backends, Anthropic generates JSON using a tool call.  We
+  ;; write the tool here, meant to be added to :tools.
+  (list
+   :name "response_json"
+   :description "Record JSON output according to user prompt"
+   :input_schema (gptel--preprocess-schema
+                  (gptel--dispatch-schema-type schema))))
+
 (cl-defmethod gptel--parse-tools ((_backend gptel-anthropic) tools)
   "Parse TOOLS to the Anthropic API tool definition spec.
 
@@ -345,7 +362,7 @@ TOOL-USE is a list of plists containing tool names, arguments and call results."
     (when (and (or (eq gptel-cache t) (memq 'message gptel-cache))
                (gptel--model-capable-p 'cache))
       (nconc (aref (plist-get (car (last full-prompt)) :content) 0)
-             '(:cache_control (:type "ephemeral"))))
+             '(:cache_control (:type "ephemeral" :ttl "1h"))))
     full-prompt))
 
 (cl-defmethod gptel--parse-buffer ((backend gptel-anthropic) &optional max-entries)
@@ -412,9 +429,10 @@ TOOL-USE is a list of plists containing tool names, arguments and call results."
           (if (stringp last-message)
               (plist-put
                (car (last prompts)) :content
-               `[(:type "text" :text ,last-message :cache_control (:type "ephemeral"))])
+               `[(:type "text" :text ,last-message
+                  :cache_control (:type "ephemeral" :ttl "1h"))])
             (nconc (aref (plist-get (car (last prompts)) :content) 0)
-                   '(:cache_control (:type "ephemeral"))))))
+                   '(:cache_control (:type "ephemeral" :ttl "1h"))))))
     prompts))
 
 (defun gptel--anthropic-parse-multipart (parts)
@@ -452,10 +470,7 @@ format."
    `(:type ,type
      :source (:type "base64"
               :media_type ,(plist-get part :mime)
-              :data ,(gptel--base64-encode media))
-     ;; TODO Make media caching a user option
-     ,@(and (gptel--model-capable-p 'cache)
-        '(:cache_control (:type "ephemeral"))))
+              :data ,(gptel--base64-encode media)))
    into parts-array
    else if (plist-get part :textfile) collect
    `(:type "text"
@@ -518,7 +533,7 @@ files in the context."
      :input-cost 3
      :output-cost 15
      :cutoff-date "2025-03")
-    (claude-opus-4-20250514
+    (claude-opus-4-1-20250805
      :description "Most capable model for complex reasoning and advanced coding"
      :capabilities (media tool-use cache)
      :mime-types ("image/jpeg" "image/png" "image/gif" "image/webp" "application/pdf")
@@ -526,6 +541,14 @@ files in the context."
      :input-cost 15
      :output-cost 75
      :cutoff-date "2025-03")
+    (claude-opus-4-20250514
+     :description "Anthropic's previous flagship model"
+     :capabilities (media tool-use cache)
+     :mime-types ("image/jpeg" "image/png" "image/gif" "image/webp" "application/pdf")
+     :context-window 200
+     :input-cost 15
+     :output-cost 75
+     :cutoff-date "2025-03")
     (claude-3-5-sonnet-20241022
      :description "Highest level of intelligence and capability"
      :capabilities (media tool-use cache)
@@ -604,8 +627,7 @@ URL `https://docs.anthropic.com/en/docs/about-claude/models#model-comparison-tab
            (lambda () (when-let* ((key (gptel--get-api-key)))
                    `(("x-api-key" . ,key)
                      ("anthropic-version" . "2023-06-01")
-                     ("anthropic-beta" . "pdfs-2024-09-25")
-                     ("anthropic-beta" . "prompt-caching-2024-07-31")))))
+                     ("anthropic-beta" . "extended-cache-ttl-2025-04-11")))))
           (models gptel--anthropic-models)
           (host "api.anthropic.com")
           (protocol "https")
